#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
  _new_app
}

teardown_file(){
  _rm_app
  _rm_server
  _reset_recipe
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
}

@test "validate app argument" {
  run $ABRA app check
  assert_failure
  assert_output --partial 'no app provided'

  run $ABRA app check DOESNTEXIST
  assert_failure
  assert_output --partial 'cannot find app'
}

@test "retrieve recipe if missing" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE"
  assert_success
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE"

  run $ABRA app check "$TEST_APP_DOMAIN"
  assert_success
  refute_output --partial '❌'

  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE"
}

@test "bail if unstaged changes and no --chaos" {
  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run $ABRA app check "$TEST_APP_DOMAIN"
  assert_failure
  assert_output --partial 'locally unstaged changes'

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "do not bail if unstaged changes and --chaos" {
  run bash -c 'echo "unstaged changes" >> "$ABRA_DIR/recipes/$TEST_RECIPE/foo"'
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run $ABRA app check "$TEST_APP_DOMAIN" --chaos
  assert_success

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "ensure recipe up to date if no --offline" {
  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 3 commits"

  run $ABRA app check "$TEST_APP_DOMAIN"
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  refute_output --partial "Your branch is behind 'origin/main' by 3 commits"

  _reset_recipe
}

@test "ensure recipe not up to date if --offline" {
  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~1
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 1 commit"

  # NOTE(d1): we can't quite tell if this will fail or not in the future, so,
  # since it isn't an important part of what we're testing here, we don't check
  # it
  run $ABRA app check "$TEST_APP_DOMAIN" --offline

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 1 commit"

  _reset_recipe
}

@test "error if missing .env.sample" {
  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/.env.sample"
  assert_success

  run $ABRA app check "$TEST_APP_DOMAIN" --chaos
  assert_failure
  assert_output --partial '.env.sample does not exist?'

  _checkout_recipe
}

@test "error if missing env var" {
  run $ABRA app check "$TEST_APP_DOMAIN"
  assert_success
  refute_output --partial '❌'

  run bash -c 'echo "NEW_VAR=foo" >> "$ABRA_DIR/recipes/$TEST_RECIPE/.env.sample"'
  assert_success

  run $ABRA app check "$TEST_APP_DOMAIN" --chaos
  assert_success
  assert_output --partial '❌'

  _checkout_recipe
}
