#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
}

teardown_file(){
  _rm_server
  _reset_recipe
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _fetch_recipe
}

teardown(){
  load "$PWD/tests/integration/helpers/common"
  _rm_app
}

@test "autocomplete" {
  run $ABRA app new --generate-bash-completion
  assert_success
  assert_output --partial "traefik"
  assert_output --partial "abra-test-recipe"

  # Note: this test needs to be updated when a new version of the test recipe is published.
  run $ABRA app new abra-test-recipe --generate-bash-completion
  assert_success
  assert_output "0.1.0+1.20.0
0.1.1+1.20.2
0.2.0+1.21.0"
}

@test "create new app" {
  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is up to date with 'origin/main'."
}

@test "create new app with version" {
  run $ABRA app new "$TEST_RECIPE" 0.1.1+1.20.2 \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" log -1
  assert_output --partial "453db7121c0a56a7a8f15378f18fe3bf21ccfdef"
}

@test "does not overwrite existing env files" {
  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success

  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_failure
  assert_output --partial 'already exists'
}

@test "bail if unstaged changes and no --chaos" {
  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_success
  assert_output --partial 'foo'

  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_failure
  assert_output --partial 'locally unstaged changes'

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "do not bail if unstaged changes and --chaos" {
  run bash -c 'echo "unstaged changes" >> "$ABRA_DIR/recipes/$TEST_RECIPE/foo"'
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_success
  assert_output --partial 'foo'

  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --chaos \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

@test "ensure recipe up to date if no --offline" {
  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 3 commits, and can be fast-forwarded."

  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is up to date with 'origin/main'."

  _reset_recipe
}

@test "ensure recipe not up to date if --offline" {
  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 3 commits, and can be fast-forwarded."

  # NOTE(d1): need to use --chaos to force same commit
  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --offline \
    --chaos \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN"
  assert_success
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" status
  assert_output --partial "Your branch is behind 'origin/main' by 3 commits, and can be fast-forwarded."

  _reset_recipe
}

@test "generate secrets" {
  run $ABRA app new "$TEST_RECIPE" \
    --no-input \
    --server "$TEST_SERVER" \
    --domain "$TEST_APP_DOMAIN" \
    --secrets
  assert_success
  assert_output --partial 'generated secrets'
  assert_output --partial 'test_pass_one'
  assert_exists "$ABRA_DIR/servers/$TEST_SERVER/$TEST_APP_DOMAIN.env"

  run $ABRA app secret ls "$TEST_APP_DOMAIN"
  assert_success
  assert_output --partial 'test_pass_one'
}
